function [t,y] = LinImpMidPointSolveSol(f,df,y0,T,h)
%
% Aufgabe 3f.
% Name: 
%
%  [T,Y] = LINIMPMIDPOINTSOLVE(F,Y0,[T0 T1],H) solves the ODE y'=f(t,y) over
%  the interval [T0 T1] with y(T0) = Y0 using the linear implicit mid-point
%  method with step size (at most) H. The input argument F should be a
%  function handle taking the arguments (t,y) and return a column vector
%  corresponding to f(t,y).

% Initialize variables
N = ceil((T(2)-T(1))/h); % number of steps
h = (T(2)-T(1))/N; % adjust step size to fit interval length
d = size(y0,1); % dimension of solution
t = linspace(T(1),T(2),N+1); % time grid
y = nan(d,N+1); % solution
y(:,1) = y0; % set initial value
fval = f(T(1),y0); % initial right hand side

% Compute solution
for j=1:N
  y(:,j+1) = LinImpMidPointStep(f,df,y(:,j),t(j),h);
end

function y = LinImpMidPointStep(f,df,y0,t0,h)
  DF = df(t0,y0);
  k1 = ( eye(size(DF)) - 0.5*h*DF )\f(t0,y0);
  % or
  % k1 = inv(eye(size(DF))-0.5*h*DF)*f(t0,y0)
  % or
  % k1 = (eye(size(DF))-0.5*h*DF)^(-1)*f(t0,y0)
  y = y0 + h*k1;
  